/* Config file loading functions. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>

#if HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen
# if HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# if HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# if HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif
 
#include <errno.h>

#include "configload.h"
#include "monitor.h"
#include "sender.h"
#include "options.h"

void loadesmconfig(char *dir,char *file);
void loadmoduleconfig(char *dir,char *file);

/* Have we loaded the config dir? */
int haveconfig = 0;

/* Adds a slash to the end of a char if needed. */
char *addslash(char *slash){
  int slashlen;

  /* If needed add a / to the end of dir. */
  slashlen = strlen(slash);
  if (slash[slashlen - 1] != '/'){
    slash = realloc(slash,slashlen + 2);
    slash[slashlen] = '/';
    slash[slashlen + 1] = '\000';
  }

  return slash;
}

/* Loads the config directory. */
int loadconfig(){
  DIR *dir_fd = NULL;
  struct dirent *tmpdirent = NULL;
  struct dirent curdirent;

  /* If needed add a / to the end of dir. */
  configdir = addslash(configdir);

  /* Open the configdir directory. */
  dir_fd = opendir(configdir);


  /* Error? */
  if (!dir_fd){
    #if HAVE_STRERROR
    fprintf(stderr,"Couldn't open configdir %s - %s\n",configdir,strerror(errno));
    #else
    fprintf(stderr,"Couldn't open configdir %s\n",configdir);
    #endif
    exit(EXIT_FAILURE);
  }

  /* Read each entry in the dir. */
  while(1){

    tmpdirent = NULL;
    tmpdirent = readdir(dir_fd);

    if (!tmpdirent)
      break;

    /* The man page for readdir says that tmpdirent might get it overwritten. Better play it
       safe and copy it to another var. */
    curdirent = *tmpdirent;

    /* Figure out what to do with the file. */
    if (curdirent.d_name[0] == '.')
      continue;
    /* Ignore save files made with emacs. */
    else if (curdirent.d_name[strlen(curdirent.d_name) - 1] == '~')
      continue;
    /* Ignore backup files from emacs. */
    else if ((curdirent.d_name[strlen(curdirent.d_name) - 1] == '#') && (curdirent.d_name[0] == '#'))
      continue;
    else if (!strcmp(curdirent.d_name,"esmconfig"))
      loadesmconfig(configdir,curdirent.d_name);
    else 
      loadmoduleconfig(configdir,curdirent.d_name);
  }

  closedir(dir_fd);


  /* Check if any senders were defined. */
  if (numsenders < 1){
    fprintf(stderr,"No senders defined. Exiting \n");
    exit(EXIT_FAILURE);
  }

  haveconfig = 1;

  return 0;
}

/* A macro that checks if a line is a comment. */
#define iscomment(line) (line[0] == '#')

/* A macro that gets a number from a line in the format x # */
#define getnum(line,tag) atoi(&line[strlen(tag)])

/* A macro that checks if a tag is in a line. Replaced by below function.
#define hastag(line,tag) !strncmp(line,tag,strlen(tag))
*/

/* A function that checks if a tag is in a line. This function used to be a macro. */
int hastag(char *line, char *tag){
  char *tmp;
  int i = 0;
  
  i = sscanf(line,"%as",&tmp);

  if (i == 1){
    if (!strcmp(tmp,tag)){
      free(tmp);
      return 1;
    }
  }
  return 0;
}

/* A function that gets a time from a line. The time can be in minutes, hours, or days */
int gettime(char *line,char *tag){
  char *tmp;
  int i = 0;

  i = sscanf(line,"%*s %as",&tmp);
 
  if (i != 1)
    return 0;

  /* Got the tag. Now figure how if we are dealing with minutes, hours or days. */
  if (strchr(tmp,'m') != NULL)
    i = atoi(tmp) * 60;
  else if (strchr(tmp,'h') != NULL)
    i = atoi(tmp) * 3600;
  else if (strchr(tmp,'d') != NULL)
    i = atoi(tmp) * 86400;
  else
    i = atoi(tmp);

  return i;
}

/* Loads a esmconfig file. */
void loadesmconfig(char *dir,char *file){
  FILE *file_fd;
  char *filename;
  char tmpline[1024];
  char *tmpline2;
  int tmp;
  
  /* The different options. */
  char strupdateinterval[] = "updateinterval";
  char strmonitortimeout[] = "monitortimeout";
  char strsendertimeout[] = "sendertimeout";
  char strmodule_kill_signal_interval[] = "module_kill_signal_interval";

  filename = malloc(strlen(dir) + strlen(file) + 1);
  filename[0] = '\000';
  filename = strcat(filename,dir);
  filename = strcat(filename,file);
  file_fd = fopen(filename,"r");

  /* Read each line and do whatever is needed. */
  while (1){

    tmpline2 = fgets(tmpline,1024,file_fd);

    /* Remove the \n at the end of tmpline. */
    if (tmpline[strlen(tmpline) - 1] == '\n')
      tmpline[strlen(tmpline) - 1] = '\000';

    if (!tmpline2)
      break;
    
    /* Is the line a comment? */
    if (iscomment(tmpline))
      continue;

    /* Parse the line. */
    if (hastag(tmpline,strupdateinterval)){
      tmp = gettime(tmpline,strupdateinterval);
      if (tmp >= 0)
	updateinterval = tmp;
      else
	fprintf(stderr,"Negative value of updateinterval in esmconfig, ignoring.\n");
    }
    else if (hastag(tmpline,strmonitortimeout)){
      tmp = gettime(tmpline,strmonitortimeout);
      if (tmp >= 0)
        monitortimeout = tmp;
      else
        fprintf(stderr,"Negative value of monitortimeout in esmconfig, ignoring.\n");
    }
    else if (hastag(tmpline,strsendertimeout)){
      tmp = gettime(tmpline,strsendertimeout);
      if (tmp >= 0)
        sendertimeout = tmp;
      else
        fprintf(stderr,"Negative value of sendertimeout in esmconfig, ignoring.\n");
    }
    else if (hastag(tmpline,strmodule_kill_signal_interval)){
      tmp = gettime(tmpline,strmodule_kill_signal_interval);
      if (tmp >= 0)
        module_kill_signal_interval = tmp;
      else
        fprintf(stderr,"Negative value of module_kill_signal_interval in esmconfig, ignoring.\n");
    }
    else{
      fprintf(stderr,"Unrecognized line %s in %s%s\n",tmpline,dir,file);
      exit(EXIT_FAILURE);
    }

  }

  fclose(file_fd);
}

/* A macro that gets a string from a line in the format x <string> The + 1 is needed to account for
 the space between the tag and the value. */
#define getstr(line,tag) (line + strlen(tag) + 1)

/* Loads a moduleconfig file. */
void loadmoduleconfig(char *dir,char *file){
  FILE *file_fd;
  char *filename;
  char tmpline[1024];
  char *tmpline2;

  /* The different options. */
  char strtitle[] = "title";
  char strcmd[] = "cmd";
  char strmonitor[] = "monitor";
  char strdmonitor[] = "dmonitor";
  char strsender[] = "sender";

  char *title = NULL;
  char *cmd = NULL;
  int type = 0;
  int pri;

  int flags = 0;

  #define TYPE_MONITOR 1
  #define TYPE_SENDER  2

  filename = malloc(strlen(dir) + strlen(file) + 1);
  filename[0] = '\000';
  filename = strcat(filename,dir);
  filename = strcat(filename,file);
  file_fd = fopen(filename,"r");

  /* Read each line and do whatever is needed. */
  while (1){

    tmpline2 = fgets(tmpline,1024,file_fd);

    /* Remove the \n at the end of tmpline. */
    if (tmpline[strlen(tmpline) - 1] == '\n')
      tmpline[strlen(tmpline) - 1] = '\000';

    if (!tmpline2)
      break;

    /* Is the line a comment? */
    if (iscomment(tmpline))
      continue;

    /* Parse the line. */
    if (hastag(tmpline,strtitle)){
      title = malloc(strlen(tmpline) + 1);
      title = strcpy(title,getstr(tmpline,strtitle));

    }
    else if (hastag(tmpline,strcmd)){
      cmd = malloc(strlen(tmpline) + 1);
      cmd = strcpy(cmd,getstr(tmpline,strcmd));

    }
    else if (hastag(tmpline,strmonitor)){
      type = TYPE_MONITOR;
    }
    else if (hastag(tmpline,strdmonitor)){
      type = TYPE_MONITOR;
      flags |= MONITOR_FLAG_DAEMON;
    }
    else if (hastag(tmpline,strsender)){
      type = TYPE_SENDER;
      pri = getnum(tmpline,strsender);
    }
    else{
      fprintf(stderr,"Unrecognized line %s in %s%s\n",tmpline,dir,file);
      exit(EXIT_FAILURE);
    }
  }

  fclose(file_fd);


  /* Save the new module in the right modulelist. */
  if ((type == TYPE_MONITOR) && title && cmd){
    MONITOR tmpmon;

    tmpmon.title = title;
    tmpmon.cmd = cmd;
    
    /* Setup the flags. */
    tmpmon.flags = flags;

    addmonitor(tmpmon);
  }
  else if ((type == TYPE_SENDER) && title && cmd){

    SENDER tmpsend;

    tmpsend.title = title;
    tmpsend.cmd = cmd;
    
    addsender(tmpsend,pri);
  }
  else{
    fprintf(stderr,"No sender or monitor ");

    if (!title)
      fprintf(stderr,"or title ");
    if (!cmd)
      fprintf(stderr,"or cmd ");

    fprintf(stderr,"tag in config file %s%s\n",dir,file);
    exit(EXIT_FAILURE);
  }
}

static char *id="@(#) $Id: configload.c,v 4.3 1999/10/09 15:42:20 pete Exp $";

/*
 * $Log: configload.c,v $
 * Revision 4.3  1999/10/09 15:42:20  pete
 * Removed a debugging printf
 *
 * Revision 4.2  1999/10/07 23:44:03  pete
 * Minor function prototype fix.
 *
 * Revision 4.1  1999/10/07 23:13:44  pete
 * Now recognizes times that have 1m for minutes, 1h for hours and 1d for
 * days.
 *
 * Revision 4.0  1999/10/07 22:56:12  pete
 * Started work on version 1.0
 *
 * Revision 3.3  1999/09/19 18:25:55  pete
 * Added support for automake
 *
 * Revision 3.2  1999/09/19 16:25:43  pete
 * Added AC_HEADER_DIRENT support.
 *
 * Revision 3.1  1999/09/19 16:11:28  pete
 * Now supports the HAVE_STRERROR #define
 *
 * Revision 3.0  1999/09/19 01:15:22  pete
 * Started work on 0.03
 *
 * Revision 2.5  1999/09/17 15:02:16  pete
 * Fixed some errors in mallocing new strings. I forgot that strlen doesn't
 * include the null terminator.
 *
 * Revision 2.4  1999/09/17 14:55:16  pete
 * Tried to remove a checkergcc warning here. Oh well. I think I'm not doing
 * anything wrong.
 *
 * Revision 2.3  1999/09/17 14:41:58  pete
 * Fixed a buffer overrun in addslash()
 *
 * Revision 2.2  1999/09/16 23:52:36  pete
 * Added dmonitor tag support.
 *
 * Revision 2.1  1999/09/16 20:26:50  pete
 * Now sets the flags var to 0 on new monitors.
 *
 * Revision 2.0  1999/09/15 16:05:51  pete
 * Started work on version 0.02
 *
 * Revision 1.30  1999/09/12 21:16:24  pete
 * Missed a debugging printf. :)
 *
 * Revision 1.29  1999/09/12 21:15:28  pete
 * Added some more error messages. The bug with the ignored lines seems to
 * have never existed. :( I can't reproduce anything. Oh well.
 *
 * Revision 1.28  1999/09/12 20:17:46  pete
 * Just found a bug. If a file doesn't end with a blank line the last line
 * seems to get ignored.
 *
 * Revision 1.27  1999/09/12 12:35:25  pete
 * Fixed bug in loadesmconfig() The string filename wasn't being initalized
 * properly.
 *
 * Revision 1.26  1999/09/11 22:22:05  pete
 * Made loadconfig ignore dot files.
 *
 * Revision 1.25  1999/09/11 22:19:51  pete
 * Bug fix, forgot to make "Unrecognized Line" errors exit for any file.
 *
 * Revision 1.24  1999/09/11 22:16:53  pete
 * Made loadconfig stop if it gets any "Unrecognized Line" errors.
 *
 * Revision 1.23  1999/09/11 19:01:43  pete
 * Added function addslash. Made loadconfig() call addslash(configdir)
 *
 * Revision 1.22  1999/09/11 18:45:44  pete
 * Now tells the user if there is a unrecognized line in a config file.
 *
 * Revision 1.21  1999/09/11 18:41:13  pete
 * Now bails out if no senders were defined.
 *
 * Revision 1.20  1999/09/11 18:36:30  pete
 * Added errorchecking on opendir
 *
 * Revision 1.19  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */
